"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typoDictionary_json_1 = __importDefault(require("./typoDictionary.json"));
const reporter = (context) => {
    const { fixer, getSource, report, RuleError, Syntax } = context;
    return {
        [Syntax.Str](node) {
            return __awaiter(this, void 0, void 0, function* () {
                const text = getSource(node);
                const ruleErrors = typoDictionary_json_1.default.flatMap((typoData) => {
                    const index = text.indexOf(typoData.word);
                    if (index === -1) {
                        return [];
                    }
                    return [
                        new RuleError(`日本語モードのまま入力された英単語: ${typoData.word} → ${typoData.correct}`, {
                            index,
                            fix: fixer.replaceTextRange([index, index + typoData.word.length], typoData.correct),
                        }),
                    ];
                });
                ruleErrors.forEach((ruleError) => {
                    if (ruleErrors.some((otherRuleError) => {
                        if (!ruleError.fix || !otherRuleError.fix) {
                            throw new Error();
                        }
                        return (ruleError.index === otherRuleError.index &&
                            ruleError.fix.range[1] < otherRuleError.fix.range[1]);
                    })) {
                        return;
                    }
                    report(node, ruleError);
                });
            });
        },
    };
};
const textlintRuleModule = {
    fixer: reporter,
    linter: reporter,
};
exports.default = textlintRuleModule;
